@@ -22,6 +22,9 @@ void Rprintf(const char *, ...);
 
 #include <glib.h>
 
+#include <signal.h> /* added by Andreas Blaette  */
+#include <sys/socket.h> /* added by Andreas Blaette */
+
 #include "globals.h"
 #include "fileutils.h"
 
