@@ -15,6 +15,18 @@
  *  WWW at http://www.gnu.org/copyleft/gpl.html).
  */
 
+/**
+ * @file
+ *
+ * This file contains functions for creating four different P-attribute components:
+ * namely CompLexiconSrt, CompCorpusFreqs, CompRevCorpus, and CompRevCorpusIdx.
+ *
+ * These are all produced by permutation of a previously encoded attribute
+ * (CompCorpus, CompLExicon, etc.)
+ */
+
+void Rprintf(const char *, ...);
+
 #include <ctype.h>
 #include <sys/types.h>
 
@@ -35,7 +47,7 @@
 #define BUFSIZE 0x10000
 
 /* seems not ever to be used? */
-char errmsg[MAX_LINE_LENGTH];
+char errmsg[CL_MAX_LINE_LENGTH];
 
 
 
@@ -51,7 +63,8 @@ static MemBlob *SortIndex;
  *
  * This function is for use with qsort().
  */
-static int scompare(const void *idx1, const void *idx2)
+static int
+scompare(const void *idx1, const void *idx2)
 {
   /* this is the way qsort(..) is meant to be used: give it void*
      and cast them to the actual type in the compare function;
@@ -65,11 +78,11 @@ static int scompare(const void *idx1, const void *idx2)
 /* note, the following functions are documented in attributes.c (a general overview)
  * in the context of the create_component() function that calls them */
 
-/* note: these functions aren't guaranteed to load the component after creating it! */
-/* (in fact, creat_rev_corpus() would run out of address space if it tried that on a large corpus) */
+/* note: these functions aren't guaranteed to load the component after creating it!
+ * (in fact, creat_rev_corpus() would run out of address space if it tried that on a large corpus) */
 
 /**
- * creates a sort index.
+ * creates a sorted index from the (already existing) lexicon index of the Attribute.
  *
  * @see create_component
  */
@@ -84,9 +97,10 @@ creat_sort_lexicon(Component *lexsrt)
   assert(lexsrt && "creat_sort_lexicon called with NULL component");
   assert(lexsrt->attribute && "attribute of component is null");
 
-  assert(comp_component_state(lexsrt) == ComponentDefined);
+  assert(comp_component_state(lexsrt) == ComponentDefined && "component is not set to Defined state");
 
-  lex = ensure_component(lexsrt->attribute, CompLexicon, 1);
+  /* make sure both the lexicon and the lexicon index components for this Att are in memory */
+  lex    = ensure_component(lexsrt->attribute, CompLexicon,    1);
   lexidx = ensure_component(lexsrt->attribute, CompLexiconIdx, 1);
 
   assert(lex != NULL);
@@ -96,14 +110,17 @@ creat_sort_lexicon(Component *lexsrt)
   assert(lexidx->data.size > 0);
   assert(lexidx->data.data != NULL);
 
+  /* read the contents of the lexidx component into the blob of the lexsrt component
+   * (note use of MALLOCED to duplicate the content). */
   if (!read_file_into_blob(lexidx->path, MALLOCED, sizeof(int), &(lexsrt->data))) {
-    fprintf(stderr, "Can't open %s, can't create lexsrt component\n", lexidx->path);
+    Rprintf("Can't open %s, can't create lexsrt component\n", lexidx->path);
     perror(lexidx->path);
     return 0;
   }
 
-  assert(lexidx->data.size == lexsrt->data.size);
-  assert(lexidx->data.nr_items == lexsrt->data.nr_items);
+  /* sanity check: make sure that the stats of the new read-in and the old read-in match */
+  assert(lexidx->data.size      == lexsrt->data.size);
+  assert(lexidx->data.nr_items  == lexsrt->data.nr_items);
   assert(lexidx->data.item_size == lexsrt->data.item_size);
 
   lexsrt->size = lexidx->size;
@@ -112,16 +129,16 @@ creat_sort_lexicon(Component *lexsrt)
   for (i = 0; i < lexsrt->data.nr_items; i++)
     lexsrt->data.data[i] = i;
 
-  /* now sort the area */
+  /* now sort the indices according to the strings they index to */
 
   SortLexicon = &(lex->data);                /* for the comparison function */
   SortIndex = &(lexidx->data);
-
   qsort(lexsrt->data.data, lexsrt->size, sizeof(int), scompare);
 
   if (write_file_from_blob(lexsrt->path, &(lexsrt->data), 1)) {
 
-    /* ok, we now have to convert the table to NETWORK order in case
+    /* ok, we now have to convert the table to NETWORK order,
+     * like in the file we just wrote, in case
      * other (later) steps rely on its format. */
 
     /* convert network byte order to native integers */
@@ -136,7 +153,7 @@ creat_sort_lexicon(Component *lexsrt)
 
 
 /**
- * creates frequency table
+ * Creates the CompCorpusFreqs component (list of type frequencies for a given p-attribute)
  *
  * @see create_component
  */
@@ -146,7 +163,7 @@ creat_freqs(Component *freqs)
   FILE *fd;
   int mc_buf[BUFSIZE];
 
-  char *corpus_fn;
+  char *corpus_fn; /* filename of CompCorpus */
 
   int i, k, ptr;
 
@@ -163,12 +180,14 @@ creat_freqs(Component *freqs)
     assert(freqs);
   }
 
+  /* load a copy of the CompLexiconIdx file into the CompCorpusFreqs data block.
+   * (NB note the use of MALLOCED to enforce operation on a *copy*, not the original... */
   if (!read_file_into_blob(lexidx->path, MALLOCED, sizeof(int), &(freqs->data))) {
-    fprintf(stderr, "Can't open %s, can't create freqs component\n", lexidx->path);
+    Rprintf("Can't open %s, can't create freqs component\n", lexidx->path);
     perror(lexidx->path);
     return 0;
   }
-  memset((char *)freqs->data.data, '\0', freqs->data.size);
+  memset((void *)freqs->data.data, '\0', freqs->data.size);
   assert(lexidx->data.size == freqs->data.size);
 
   freqs->size = lexidx->size;
@@ -176,14 +195,13 @@ creat_freqs(Component *freqs)
   corpus_fn = component_full_name(freqs->attribute, CompCorpus, NULL);
   assert(corpus_fn != NULL);
 
-  if ((fd = fopen(corpus_fn, "r")) == NULL) {
-    fprintf(stderr, "makecomps:creat_freqs(): Couldn't open corpus %s\n", corpus_fn);
+  if ((fd = fopen(corpus_fn, "rb")) == NULL) {
+    Rprintf("CL makecomps:creat_freqs(): Couldn't open corpus %s\n", corpus_fn);
     perror(corpus_fn);
     exit(2);
   }
 
   /* do the counts */
-
   do {
     i = fread(&mc_buf[0], sizeof(int), BUFSIZE, fd);
     for ( k = 0; k < i; k++) {
@@ -191,18 +209,14 @@ creat_freqs(Component *freqs)
       if ((ptr >= 0) && (ptr < freqs->size))
         freqs->data.data[ptr]++;
       else
-        fprintf(stderr, ";;; makecomps:creat_freqs(): WARNING: index %d out of range\n", ptr);
+        Rprintf("CL makecomps:creat_freqs(): WARNING: index %d out of range\n", ptr);
     }
   } while (i == BUFSIZE);
   fclose(fd);
 
-  /* first, we write the table to the file in order to convert it from
-     host to network format. */
-
+  /* first, we write the table to the file in order to convert it from host to network format. */
   if (write_file_from_blob(freqs->path, &(freqs->data), 1)) {
-
-    /* ok, we now have to convert the table to NETWORK order in case
-     * other steps rely on its format. */
+    /* ok, we now have to convert the table to NETWORK order in caseother steps rely on its format. */
 
     /* convert network byte order to native integers */
     for (ptr = 0; ptr < freqs->size; ptr++)
@@ -216,36 +230,37 @@ creat_freqs(Component *freqs)
 
 
 /**
- * creates reversed corpus
+ * Creates a reversed corpus component.
+ *
+ * This function should only be invoked by the makeall tool (via create_component()),
+ * which must make sure that the lexicon and (possibly) compressed token stream have been
+ * created by now, so CL access to the token stream works.
  *
  * @see create_component
+ * @see makeall_do_attribute
+ * @return  number of passes made through the corpus.
  */
 int
 creat_rev_corpus(Component *revcorp)
 {
-
   Component *freqs;
   int cpos = 0, f, id, ints_written, pass;
 
   int datasize;
   int primus, secundus, lexsize, buf_used;
   int *buffer;
-  size_t bufsize;
-  int **ptab;                        /* pointers into <buffer> */
+  size_t bufsize;                     /* size of buffer (measured in number of 4-byte integers) */
+  int **ptab;                         /* pointers into <buffer> */
   int *ptr;
 
   FILE *revcorp_fd;
-  Attribute *attr;                /* the attribute we're working on */
-
-  /* this function should only be invoked by the makeall tool (via create_component()),
-     which must make sure that the lexicon and (possibly) compressed token stream have been
-     created by now, so CL access to the token stream works (see do_attribute()<makeall.c>) */
+  Attribute *attr;                    /* the attribute we're working on */
 
   assert(revcorp != NULL);
   assert(revcorp->path != NULL);
   assert(revcorp->data.data == NULL); /* so REVCORP is unloaded */
 
-  attr = revcorp->attribute;        /* need the attribute handle to use CL functions */
+  attr = revcorp->attribute;          /* need the attribute handle to use CL functions */
 
   /* get the frequency table to compute offsets and fill buffer */
   freqs = ensure_component(attr, CompCorpusFreqs, 1);
@@ -254,7 +269,7 @@ creat_rev_corpus(Component *revcorp)
   assert(freqs->corpus == revcorp->corpus); /* gotta be kidding ... */
 
   lexsize = cl_max_id(attr);        /* this is the number of lexicon entries for this attribute */
-  ptab = (int **) cl_malloc(sizeof(int *) * lexsize); /* table of pointers into <buffer> */
+  ptab = (int **) cl_malloc(sizeof(int *) * ((size_t) lexsize)); /* table of pointers into <buffer> */
 
   /* determine REVCORP size (== number of tokens) */
   datasize = cl_max_cpos(attr);
@@ -264,23 +279,24 @@ creat_rev_corpus(Component *revcorp)
   if (datasize < bufsize) {
     bufsize = datasize;                /* shrink buffer if full size isn't needed */
   }
-  buffer = cl_malloc(4 * bufsize); /* allocate buffer */
+  buffer = cl_malloc(4 * bufsize); /* allocate buffer with 4 bytes per integer */
 
   /* open REVCORP data file for writing */
-  if ((revcorp_fd = fopen(revcorp->path, "w")) == NULL) {
+  if ((revcorp_fd = fopen(revcorp->path, "wb")) == NULL) {
     perror(revcorp->path);
     exit(1);
   }
 
-  /* NEW multi-pass algorithm.
+  /*
+     NEW multi-pass algorithm.
      In each pass through the corpus, occurrences of lex ID <primus> are directly written
      to the REVCORP file, and occurrences of IDs <primus>+1 ... <secundus> are stored in
      <buffer> (which has room for <bufsize> INTs), then written to REVCORP file.
-     */
+  */
 
   if (cl_debug) {
-    fprintf(stderr, "\nCreating REVCORP component as '%s' ... \n", revcorp->path);
-    fprintf(stderr, "Size = %d INTs,  Buffer Size = %ld INTs\n", datasize, bufsize);
+    Rprintf("\nCreating REVCORP component as '%s' ... \n", revcorp->path);
+    Rprintf("Size = %d INTs,  Buffer Size = %ld INTs\n", datasize, bufsize);
   }
 
   primus = 0;
@@ -288,13 +304,12 @@ creat_rev_corpus(Component *revcorp)
   pass = 0;                        /* count pass for debugging output */
   while (primus < lexsize) {
 
-    /* see how many IDs fit into the buffer in one pass */
+    /* see how many lexicon IDs fit into the buffer in one pass */
     buf_used = 0;                /* how many buffer entries are used */
     for (secundus = primus + 1; secundus < lexsize; secundus++) {
       /* increment secundus to fit as many IDs as possible into the buffer for this pass */
       f = cl_id2freq(attr, secundus);
       if (buf_used + f > bufsize) {
-        secundus--;
         break;
       }
       else {
@@ -302,16 +317,17 @@ creat_rev_corpus(Component *revcorp)
         buf_used += f;
       }
     }
+    secundus--; /* this is the last valid lexicon ID we're indexing in this pass */
 
     pass++;
     if (cl_debug) {
       double perc = (100.0 * secundus) / lexsize;
-      fprintf(stderr, "Pass #%-3d (%6.2f%c complete)\n", pass, perc, '%');
+      Rprintf("CL makecomps: Pass #%-3d (%6.2f%c complete)\n", pass, perc, '%');
     }
 
     for (cpos = 0; cpos < datasize; cpos++) {
-      id = cl_cpos2id(attr, cpos);          /* lex. ID of token found at <cpos> */
-      assert((id >= 0) && (id < lexsize) && "Lexicon ID out of range. Abort.");
+      id = cl_cpos2id(attr, cpos);          /* id contains the lex. ID of the token found at <cpos> */
+      assert((id >= 0) && (id < lexsize) && "CL makecomps: Lexicon ID out of range. Abort.");
       if (id == primus) {
         NwriteInt(cpos, revcorp_fd); /* converts to network byte order */
         ints_written++;
@@ -323,10 +339,10 @@ creat_rev_corpus(Component *revcorp)
 
     /* check pointers (i.e. observed frequencies vs. data from FREQS component) */
     ptr = buffer;
-    for (id = primus + 1; id < secundus; id++) {
+    for (id = primus + 1; id <= secundus; id++) {
       ptr += cl_id2freq(attr, id);
       if (ptr != ptab[id]) {
-        fprintf(stderr, "Pointer inconsistency for id=%d. Aborting.\n", id);
+        Rprintf("CL makecomps: Pointer inconsistency for id=%d. Aborting.\n", id);
         exit(1);
       }
     }
@@ -337,14 +353,15 @@ creat_rev_corpus(Component *revcorp)
 
     /* now start next pass ... */
     primus = secundus + 1;
-  }                                             /* endwhile (primus < lexsize) */
+
+  } /* endwhile (primus < lexsize) */
 
   /* we're done: close REVCORP filehandle */
   fclose(revcorp_fd);
 
   /* finally, check amount of data read/written vs. expected */
   if ((ints_written != cpos) || (ints_written != datasize)) {
-    fprintf(stderr, "Data size inconsistency: expected=%d, read=%d, written=%d.\n", datasize, cpos, ints_written);
+    Rprintf("CL makecomps: Data size inconsistency: expected=%d, read=%d, written=%d.\n", datasize, cpos, ints_written);
     exit(1);
   }
 
@@ -353,10 +370,10 @@ creat_rev_corpus(Component *revcorp)
   cl_free(ptab);
 
   /*   (void) load_component(attr, CompRevCorpus);  */
-  /* new in 2.2.b79: a newly created component isn't loaded automatically, in order to
-     avoid running out address space for large corpora [status should be ComponentUnloaded] */
+  /* a newly created component isn't loaded automatically, in order to
+     avoid running out of address space for large corpora [status should be ComponentUnloaded] */
 
-  /* returns number of passes now */
+  /* return number of passes */
   return pass;
 }
 
@@ -365,6 +382,7 @@ creat_rev_corpus(Component *revcorp)
  * creates index for reversed corpus
  *
  * @see create_component
+ * @return  Returns 1.
  */
 int
 creat_rev_corpus_idx(Component *revcidx)
@@ -380,6 +398,7 @@ creat_rev_corpus_idx(Component *revcidx)
   assert(freqs != NULL);
   assert(freqs->corpus == revcidx->corpus);
 
+  /* directly manipulate the MemBlob internals of the new component ... */
   revcidx->data.size = freqs->data.size;
   revcidx->data.item_size = SIZE_INT;
   revcidx->data.nr_items = freqs->data.nr_items;
@@ -390,30 +409,28 @@ creat_rev_corpus_idx(Component *revcidx)
   revcidx->data.fsize = 0;
   revcidx->data.offset = 0;
 
+  /* equivalent to using MALLOCED when calling one of the MemBlob functions... */
   revcidx->data.data = (int *)cl_malloc(sizeof(int) * revcidx->data.nr_items);
   memset(revcidx->data.data, '\0', revcidx->data.size);
   revcidx->size = revcidx->data.nr_items;
 
   sum = 0;
   for (k = 0; k < freqs->size; k++) { /* for each entry in freqs ... */
+    i = ntohl(freqs->data.data[k]);    /* i = the frequency of type[k] */
 
-    i = ntohl(freqs->data.data[k]);    /* the frequency of word[k] */
-
-    /* the start of word[k] is the sum of freqs of words[i<k] */
+    /* the startpoint in the reversed index of the entries for type[k] is the sum of freqs of all types whose ID is less than k */
     revcidx->data.data[k] = htonl(sum);
 
     sum += i;          /* compute the sum for the next word */
   }
 
-  /* sum should be the number of words in the corpus, that
+  /* sum should be the number of tokens in the corpus, that
      is, the length of the corpus file / sizeof(int). Check this. */
 
   /* WE DO NOT CONVERT the table from host to network order while
-   * writing it, since it's already been created in network order!!!
-   * */
-
+   * writing it, since it's already been created in network order!!! */
   if (write_file_from_blob(revcidx->path, &(revcidx->data), 0) == 0) {
-    fprintf(stderr, "Can't open %s for writing", revcidx->path);
+    Rprintf("CL makecomps: Can't open %s for writing", revcidx->path);
     perror(revcidx->path);
     exit(2);
   }
